import io

import requests
import base64
import os
from django.conf import settings
from django.core.files.storage import Storage
from django.core.files import File
from django.utils.deconstruct import deconstructible


@deconstructible
class ImageStorage(Storage):

    def __init__(self):
        self.option = settings.CUSTOM_STORAGE_OPTIONS

    def headers(self):
        token = self.option['token']
        headers = {'Authorization': 'Bearer {}'.format(token)}
        return headers

    def url(self, name):
        url = self.option['url']
        rc = requests.get(url + '/' + name, headers=self.headers())
        if rc.status_code != 200:
            return ''
        return rc.json()['url']

    def _open(self, name, mode='rb'):
        if self.exists(name):
            r = requests.get(self.url(name), headers=self.headers())
            return File(io.BytesIO(r.content), name)

    def _save(self, name, content):
        extension = os.path.splitext(name)[1][1:]
        assert extension in ['jpg', 'png'], 'Not allowed extension: {}'.format(extension)
        content_type = {'jpg': 'image/jpeg',
                        'png': 'image/png'}[extension]
        content = content.open('rb').read()
        url = self.option['url']
        rc = requests.post(url,
                           files={'content': (None, content, content_type), 'extension': (None, extension, 'text/plain')},
                           headers=self.headers())
        assert rc.status_code == 201, 'Bad response code: {}'.format(rc.status_code)
        new_name = rc.json()['id']
        return new_name

    def exists(self, name):
        return self.url(name) != ''

    def size(self, name):
        file = self.open(name)
        return file.size




import requests
from django import template
from django.conf import settings

register = template.Library()

@register.filter(name='sdh_cdn_image')
def sdh_cdn_image(value, str_arg):
    kwargs = {}
    for arg in str_arg.split(','):
        [key, val] = arg.split('=')
        kwargs[key] = val
    width = kwargs.get('width', 100)
    height = kwargs.get('height', 100)
    cropping = kwargs.get('cropping', 0)

    url = settings.CUSTOM_STORAGE_OPTIONS['url']
    token = settings.CUSTOM_STORAGE_OPTIONS['token']
    headers = {'Authorization': 'Bearer {}'.format(token)}
    url = '{}/{}?width={}&height={}&cropping={}'.format(url, value.name, width, height, cropping)
    rc = requests.get(url, headers=headers)
    if rc.status_code != 200:
        return ''
    return rc.json()['url']






from setuptools import find_packages, setup

version = '1.1.9'

setup(
    name='sdh.cdn',
    version=version,
    url='https://bitbucket.org/sdh-llc/sdh-cdn/',
    author='Software Development Hub LLC',
    author_email='dev-tools@sdh.com.ua',
    description='Image CDN engine',
    license='BSD',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['sdh'],
    eager_resources=['sdh'],
    include_package_data=True,
    entry_points={},
    install_requires=['Django>=1.8', ],
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



import importlib
from types import ModuleType
from django.core.management.base import BaseCommand, CommandError
from django.db.models.fields.files import ImageFileDescriptor

def get_class(path):
    class_path = path.split('.')
    if len(class_path) == 1:
        raise CommandError('Please set the dot separated path to Model Class (e.g. app.module.Class)')
    module = '.'.join(class_path[:-1])
    class_name = class_path[-1]
    try:
        _class = getattr(importlib.import_module(module), class_name)
    except ModuleNotFoundError as e:
        raise CommandError('ModuleNotFoundError: %s' % e)
    except AttributeError as e:
        raise CommandError(e)
    if isinstance(_class, ModuleType):
        raise CommandError('%s is module, not a class' % path)
    return _class


class Command(BaseCommand):
    help = 'Migrates DB stored images to another storage'

    def add_arguments(self, parser):
        parser.add_argument('-m', '--model', help='Model class')
        parser.add_argument('-f', '--field', help='Image field')
        parser.add_argument('-o', '--old', help='Old storage class')

    def handle(self, *args, **options):
        if not options['model']:
            raise CommandError('Please set the Model Class Path (-m, --model option)')
        if not options['old']:
            raise CommandError('Please set the Old Storage Class (-o, --old option)')
        if not options['field']:
            raise CommandError('Please set the Image Field (-f, --field option)')

        model = get_class(options['model'])
        old = get_class(options['old'])()

        try:
            image_field = getattr(model, options['field'])
        except AttributeError:
            raise CommandError('%s is not a %s class field' % (options['field'], options['model']))
        if image_field.__class__ != ImageFileDescriptor:
            raise CommandError('Field %s.%s is not a image field' % (options['model'], options['field']))

        for record in model.objects.all():
            self.stdout.write(str(record))
            image = getattr(record, image_field.field.name)
            name = image.name
            extension = name.split('.')[-1]
            if not extension in ['jpg', 'png']:
                extension = old.url(image.name).split('.')[-1]
                name = '{}.{}'.format(name, extension)
            if old.exists(image.name):
                file = old.open(image.name)
            else:
                self.stdout.write('  {}  not exist in storage {}'.format(image.name, options['old']))
                continue
            if image.field.storage.exists(name):
                new_name = 'Already saved'
            else:
                new_name = image.field.storage.save(name, file)
                image.name = new_name
            self.stdout.write('  {}  >>>  {}'.format(name, new_name))
            record.save()

        self.stdout.write(self.style.SUCCESS('Success'))
